admixtureRate <- function(segmentwithratio, mcmc = 10000, burnin = 5000, p = 0.01, weight=FALSE){
  
  ###### Written by Mengjie Chen, Nov 2012
  ###### Last time modified in June, 2013  
  ###### Estimating admixture rate of normal cells by a Bayesian finite mixture model
  
  rdirichlet <- function(n, alpha){
    k <- length(alpha)
    r <- matrix(0, nrow=n, ncol=k)
    for(i in 1:k) {
      r[,i] <- rgamma(n, alpha[i], 1)
    }
    r <- matrix(mapply(function(r, s) {return (r/s)}, r, rowSums(r)), ncol=k)
    return (r)
  }
  
  if(class(segmentwithratio) != "GRanges")
    stop("Wrong input! Please check the manuals.")
  if(!"ratio"%in%colnames(elementMetadata(segmentwithratio)))
    stop("No somatic ratio! Please check the manuals.")  
  inputseg <- as(segmentwithratio, "data.frame")
  inputseg <- inputseg[(inputseg$end-inputseg$start)/1000>1000, ]
  aa <- inputseg$ratio*2
  if(length(unique(round(aa)))==1) stop("SomatiCA cannot estimate admixture rate for samples without obvious copy number changes.")	
  subseg <- inputseg[aa < 3.5, ]
  mixture <- aa[aa < 3.5]

  ######## initialization ######  
  submix <- mixture 
  
  suppressPackageStartupMessages(require(rebmix))
  len <- length(submix)
  v <- c(as.integer(1 + log2(sum(len))), ## Minimum v follows the Sturges rule.
         as.integer(2 * sum(len)^0.5)) ## Maximum v follows the RootN rule.  
  N <- as.integer(log(v[2] / (v[1] + 1)) / log(1 + 1 / v[1]))
  K <- c(v[1], as.integer((v[1] + 1) * (1 + 1 / v[1])^(0:N)))
  suppressMessages(ax <- REBMIX(Dataset = list(submix = as.data.frame(submix)), 
                                Preprocessing = c("histogram"), Criterion = c("AIC"), 
                                Variables = "continuous", pdf = "normal", K = K))
  ncenters <- as.numeric(ax$summary[8])
  if(ncenters <= length(unique(round(mixture)))){
      if(0%in%sort(unique(round(mixture))) | min(mixture) < 1){
          centers <- sort(unique(c(0, round(mixture))))   
          integer_centers <- centers
          assigned <- unlist(lapply(mixture, function(x){ which.min(abs(centers-x)) }))
          if(!any(assigned==1)){ assigned[which.min(mixture)] <- 1 }
      }
      else if(any(aa>2.8)){
          atleast <- 0.4
          maxlevel <- 4
          minlevel <- min(floor(mixture))
          assigned.centers <- atleast*2 + (1-atleast)*c(minlevel:maxlevel)       
          centers <- integer_centers <- c(minlevel:maxlevel)
          assigned <- unlist(lapply(mixture, function(x){ which.min(abs(assigned.centers-x)) })) 
          if(!any(assigned==length(centers))){ assigned[which.max(mixture)] <- length(centers) }
          if(!any(assigned==1)){ assigned[which.min(mixture)] <- 1 }
      }
      else{
          centers <- sort(unique(round(mixture)))
          integer_centers <- centers
          assigned <- unlist(lapply(mixture, function(x){ which.min(abs(centers-x)) }))  
      }
  }
  else{
      histogram_centers <- round(as.numeric(sort(ax$Theta[[1]][2, ])), 3)
      if(min(diff(histogram_centers)) < 0.25) {
          histogram_centers <- histogram_centers[- (which(diff(histogram_centers) < 0.25)+1)] 
          ncenters <- length(histogram_centers)
      }
      other_centers <- histogram_centers[-which.min(abs(histogram_centers-2))]
      if(any(abs(other_centers-2) < 0.3)) {
          histogram_centers <-  sort(c(other_centers[-which(abs(other_centers-2) < 0.3)],
                              histogram_centers[which.min(abs(histogram_centers-2))]))
          ncenters <- length(histogram_centers)
      }
      if(length(other_centers[other_centers>2])>2) {
          aa <- length(histogram_centers[histogram_centers>2])-2
          histogram_centers <- histogram_centers[-c(length(histogram_centers):(length(histogram_centers)-aa+1))]
      }
      if(length(histogram_centers) <= 2) {
          centers <- sort(unique(round(mixture)))     
          integer_centers <- centers
          assigned <- unlist(lapply(mixture, function(x){ which.min(abs(centers-x)) }))
      }
      else {
          if(any(histogram_centers>2)) pos_normal <- min(which(histogram_centers>2))    
          else pos_normal <- ncenters + 1         
          if(pos_normal>3) integer_centers <- c(0:(ncenters-pos_normal+3))
          else if(pos_normal==3) integer_centers <- c(0:(ncenters-1))
          else if(pos_normal==2) integer_centers <- c(floor(histogram_centers[1]), 2:(ncenters))
          else if(pos_normal==1) integer_centers <- c(1:(ncenters+2))
          assigned <- unlist(lapply(mixture, function(x){ which.min(abs(histogram_centers-x)) }))   
          centers <- histogram_centers
      }      
  }
  
  mu <- NULL  
  n <- length(centers)
  if(n==1) stop("SomatiCA cannot estimate admixture rate for samples without obvious copy number changes.")	
  
  group <- assigned  
  group[sample(length(mixture), round(1/5*length(group)))] <- sample(n, round(1/5*length(group)), prob=rep(1/n, n), replace=T)
  nstarts <- table(assigned)
  
  ######## MCMC #######
  tau2 <- 0.01
  epsi2 <- 0.01
  s <- c(1:n)
  trace <- NULL
  count <- matrix(0, nrow=n, ncol=length(mixture))
  mu0 <- rep(0, n)
  
  for (i in 1:mcmc){
    
    ll <- paste("x", 1:n, "<- sum(mixture[group==", 1:n, "])", sep="")
    eval(parse(text=ll))
    ll <- paste("n", 1:n, "<- length(mixture[group==", 1:n, "])", sep="")
    eval(parse(text=ll))
    ll <- paste("mu", 1:n, "<- rnorm(1, mean=(epsi2*centers[", 1:n, "]+tau2*x", 1:n, ")/(epsi2+tau2*n", 1:n, "), sd = sqrt(epsi2*tau2/(epsi2*n", 1:n, "+tau2)))", sep="")
    eval(parse(text=ll))
    ll <- paste("mu", 1:n, sep="")
    tt <- paste("mu <- c(", paste(ll, collapse=", "), ")", sep="")
    eval(parse(text=tt))
    k <- ceiling(runif(1)*length(mixture))  
    phos <- rdirichlet(1, nstarts+1/n)
    if(length(phos)!=n) phos <- rdirichlet(1, table(assigned))
    proposed.Tk <- sample(s, 1, prob=phos)
    p.T <- dnorm(mixture[k], mean=mu[group[k]], sd=sqrt(epsi2), log=T)
    p.proposedT <- dnorm(mixture[k], mean=mu[proposed.Tk], sd=sqrt(epsi2), log=T)
    if(p.proposedT > p.T) accept <- 1
    else  accept <- exp(p.proposedT-p.T)
    u <- runif(1)
    if(u <= accept) group[k]=proposed.Tk
    trace <- c(trace, sum(dnorm(mixture, mean=mu[group], sd=sqrt(epsi2), log=T)))	
    if(i > burnin){
      for(j in 1:n){
        flag = group==j
        count[j, flag==1]=count[j, flag==1]+1
      }
      mu0 <-  as.numeric(mu0) + as.numeric(mu)	
    }
   # nstarts <- table(group)
  }
  
  posterior_mu <- mu0/(mcmc-burnin)
  ave <- apply(count, 2, function(y){y/sum(y)})
  tt <- c(1:length(mixture))[apply(ave, 2, max) > 1-p]
  group <- apply(ave, 2, which.max)
  ally <- cbind(mixture, group)
  y <- ally[integer_centers[group]%in%c(0, 1, 3) & c(1:length(ally[, 1])) %in% tt & !subseg$seqnames %in% c("chrX", "chrY", "chrM"), ]
  if(nrow(y) <= 5){
      y <- ally[integer_centers[group]%in%c(0, 1, 3) & !subseg$seqnames %in% c("chrX", "chrY", "chrM"), ]
  }
  
  if(weight==FALSE) w <- rep(1, length(y[, 1]))
  else{
      w <- c(subseg$end-subseg$start)[integer_centers[group]%in%c(0, 1, 3) & c(1:length(subseg$start)) %in% tt & !subseg$seqnames %in% c("chrX", "chrY", "chrM")]/1000
      w <- log(w+1)
  }
  #adjust <- round(mu[which.min(abs(mu-2))], 3)
  if(is.vector(y)){
    fr <- function(c){
        sum(w*((1-c)*2*integer_centers[y[2]]/4 + c - y[1]/2)^2)
    }
  }
  else{
    fr <- function(c){
        sum(w*((1-c)*2*integer_centers[y[, 2]]/4 + c - y[, 1]/2)^2)
    }
  }    	
  opt.admix <- optimize(fr, c(0, 1))$minimum	
  admix <- round(opt.admix, 3)
  if(admix > 0.8) admix <- 1 - admix
  if(admix == 1) admix <- 0
  result <- list(admix = admix, mu = posterior_mu, cluster = cbind(round(mixture/2, 3), integer_centers[group]))
  return(result)	
	
}